!=============================================================================!
!=============================================================================!
!                HYDRO Boundary Condition Passing Among Processors
!=============================================================================!
!=============================================================================!
!
!------------------------------------------------------------------------------
!                             Copyright,  2009
!                                Greg Howes
!------------------------------------------------------------------------------
!  NOTE: Boundary Conditions for the whole simulation box are periodic
!------------------------------------------------------------------------------
module hydro_bcs
  implicit none
  private

  public :: pass_bcs

contains
!------------------------------------------------------------------------------
!                           Greg Howes, 2009
!------------------------------------------------------------------------------
! Pass required information at boundaries of subdomains
  subroutine pass_bcs
    use hydro_mpi_var, only: nproc,iproc,proc0,ierror
    use hydro_var, only: qq
    use hydro_param, only: nx,ny
    use hydro_grid, only: lnx0,lnx1,iproc_ix
   implicit none
   include 'mpif.h'                  !Include MPI library variables
   integer :: sproc, rproc           !Sending and receiving processors
   integer :: sinx,rinx              !ix index to pass first
   integer, dimension (MPI_STATUS_SIZE) :: status
   !DEBUG
!   real, dimension(:),pointer :: rtmp,stmp !Temp receive and send arrays 
!   real, dimension(:),pointer  :: t1,t2 !Temp receive and send arrays 
!   real :: tt1, tt2

   !Pass periodic information locally in the y-dimension
!   if (proc0) write(*,'(a)')'BC: Pass ny BCs' 
   qq(:,lnx0:lnx1,-1)=qq(:,lnx0:lnx1,ny-1)
   qq(:,lnx0:lnx1,ny)=qq(:,lnx0:lnx1,0)
   
   !Pass the boundary information to the neighboring processor
   !NOTE: First pass lower boundary, then upper boundary
   !NOTE: mod's below handle wrap around from iproc=(nproc-1) to iproc=0
   !Lower boundary information-------------------------------------------------
!   if (proc0) write(*,'(a)')'BC: Lower' 
   rinx=mod(lnx0-1 + nx, nx) !inx needed by this processor
   sinx=lnx1                 !inx needed by next higher processor 
   !Get processor numbers with that have rinx and that need sinx
   rproc=iproc_ix(rinx)
   sproc=mod(iproc+1,nproc)

   !DEBUG
!   if (.true.) then
!      write (*,'(i3,5(a,i4))')iproc,': s=',sinx,' sproc=',sproc,'; rr=',rinx,' rl=',lnx0-1,' rproc=',rproc
!   endif
   
   !DEBUG
!   allocate(t1(1:nx));t1(:)=real(iproc)
!   allocate(t2(1:nx));t2(:)=0.
!   tt1=3.14; tt2=0.
   
   !DEBUG
!   if (.false.) then
!      if (proc0) then
!         write(*,'(a,i6)')'ERROR: Number of processors must be even: nproc= ',nproc
!         write(*,'(a)')   'Finishing HYDRO==================================' 
!      endif
!      call mpi_finalize (ierror)
!      stop
!   endif

   !Pass boundary information
   !NOTE:To avoid deadlock, the even iproc always passes first
   if (mod(iproc,2) .eq. 0) then
!      write (*,'(i3,3(a,i4))')iproc,':SEND: ', iproc,' to ',sproc,' ix=',sinx
 !     stmp=>qq(1,sinx,:)
!      write (*,'(a,f6.4,2(a,i4))')'SENDING tt1=', tt1,' From ',iproc,' to ' , sproc
!      call mpi_send(tt1, 1, MPI_REAL8, sproc, 0, mpi_comm_world, ierror)
 !     write (*,'(a,f6.4)')'SENT tt1=', tt1
!      call mpi_send(t1, size(t1), MPI_REAL8, sproc, 0, mpi_comm_world, ierror)
!      call mpi_send(qq(1,sinx,:), size(qq(1,sinx,:)), MPI_REAL8, sproc, 0, mpi_comm_world, ierror)
!      call mpi_barrier(mpi_comm_world); write (*,'(i3,a)')iproc,'BARRIER'
!      write (*,'(i3,3(a,i4))')iproc,':RECV: ', rproc,' to ',iproc,' ix=',lnx0-1
      call mpi_send(qq(:,sinx,:), size(qq(:,sinx,:)), MPI_REAL8, sproc, 0, mpi_comm_world, ierror)
      call mpi_recv(qq(:,lnx0-1,:), size(qq(:,lnx0-1,:)), MPI_REAL8, rproc, 0, mpi_comm_world, status, ierror)
   else
!      rtmp=>qq(1,lnx0-1,:)
!      write (*,'(i3,3(a,i4))')iproc,':RECV: ', rproc,' to ',iproc,' ix=',lnx0-1
!      write (*,'(a,f6.4,3(a,i4))')'RECVING tt2=', tt2,' From ',rproc,' to ' , iproc
!      call mpi_recv(tt2, 1, MPI_REAL8, rproc, 0, mpi_comm_world, status, ierror)
!      call mpi_recv (a, size(a), mpireal, src, tagp, communicator,status, ierror)
!      write (*,'(a,f6.4,3(a,i4))')'RECVED tt2=', tt2
!      call mpi_recv(t2, size(t2), MPI_REAL8, rproc, 0, mpi_comm_world, ierror)
!      call mpi_recv(qq(1,lnx0-1,:), size(qq(1,lnx0-1,:)), MPI_REAL8, rproc, 0, mpi_comm_world, ierror)
!      call mpi_barrier(mpi_comm_world);write (*,'(i3,a)')iproc,'BARRIER'
!      write (*,'(i3,3(a,i4))')iproc,':SEND: ', iproc,' to ',sproc,' ix=',sinx
      call mpi_recv(qq(:,lnx0-1,:), size(qq(:,lnx0-1,:)), MPI_REAL8, rproc, 0, mpi_comm_world, status, ierror)
      call mpi_send(qq(:,sinx,:), size(qq(:,sinx,:)), MPI_REAL8, sproc, 0, mpi_comm_world, ierror)
   endif
!   if (proc0) write(*,'(a)')'STOPPING'  ; call mpi_finalize (ierror);  stop

   !Upper boundary information-------------------------------------------------
!   if (proc0) write(*,'(a)')'BC: Upper' 
   rinx=mod(lnx1+1, nx)      !inx needed by this processor
   sinx=lnx0                 !inx needed by next higher processor 
   !Get processor numbers with that have rinx and that need sinx
   rproc=iproc_ix(rinx)
   sproc=mod(iproc-1+nproc,nproc)
   !Pass boundary information
   !NOTE:To avoid deadlock, the even iproc always passes first
   if (mod(iproc,2) .eq. 0) then
      call mpi_send(qq(:,sinx,:), size(qq(:,sinx,:)), MPI_REAL8, sproc, 0, mpi_comm_world, ierror)
      call mpi_recv(qq(:,lnx1+1,:), size(qq(:,lnx1+1,:)), MPI_REAL8, rproc, 0, mpi_comm_world, status, ierror)
   else
      call mpi_recv(qq(:,lnx1+1,:), size(qq(:,lnx1+1,:)), MPI_REAL8, rproc, 0, mpi_comm_world, status, ierror)
      call mpi_send(qq(:,sinx,:), size(qq(:,sinx,:)), MPI_REAL8, sproc, 0, mpi_comm_world, ierror)
   endif

  end subroutine pass_bcs
!------------------------------------------------------------------------------


end module hydro_bcs
